/*
 * This file is a part of morm
 * and is distributed under the 2-Clause BSD licence.
 * Author: Tomasz Sowa <t.sowa@ttmath.org>
 */

/*
 * Copyright (c) 2018, Tomasz Sowa
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in the
 * documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 */

#ifndef headerfile_morm_model
#define headerfile_morm_model

#include <string>

#include "textstream/textstream.h"
#include "modelconnector.h"




namespace morm
{


class Model
{
public:

	void set_connector(ModelConnector & connector);

	/*
	 * map fields to names
	 *
	 *
	 */
	virtual void map_fields() = 0;


	/*
	 *
	 * we can use the object for a different purpose than database (e.g. json)
	 * so let the table_name be non pure-virtual
	 *
	 */
	virtual void table_name(PT::TextStream & stream);


	virtual void to_text(PT::TextStream & stream);
	virtual void to_text(std::string & str);
	virtual std::string to_text();
	virtual std::string to_string();

	virtual void generate_insert_query(PT::TextStream & stream);
	virtual void insert();

	virtual void generate_update_query(PT::TextStream & stream);
	virtual void update();

	virtual void set_object_exists(bool object_exists);

	virtual bool object_exists();
	virtual bool found();

	// set object to default values
	virtual void clear();


protected:

	ModelConnector * model_connector;
	bool morm_object_exists;

	Model();
	virtual ~Model();

	virtual void before_select();
	virtual void before_insert();
	virtual void before_update();

	virtual void after_select();
	virtual void after_insert();
	virtual void after_update();

	template<typename FieldValue>
	void field(const wchar_t * field_name, FieldValue & field_value, bool insertable = true, bool updatable = true, bool is_primary_key = false)
	{
		if( model_connector )
		{
			model_connector->field(field_name, field_value, insertable, updatable, is_primary_key);
		}
	}

	template<typename ModelClass>
	void field_list(const wchar_t * field_name, std::list<ModelClass> & field_list, bool insertable = true, bool updatable = true, bool is_primary_key = false)
	{
		if( model_connector )
		{
			model_connector->field_list(field_name, field_list, insertable, updatable, is_primary_key);
		}
	}

	template<typename FieldValue>
	void get_last_sequence(const wchar_t * sequence_table_name, FieldValue & field_value)
	{
		if( model_connector )
		{
			model_connector->get_last_sequence(sequence_table_name, field_value);
		}
	}

	//void field(const wchar_t * field_name, Model & field, bool insertable = true, bool updatable = true);

	template<typename ModelClass> friend class Finder;
	friend class ModelConnector;
};

} // namespace

#endif
