/*
 * This file is a part of morm
 * and is distributed under the 2-Clause BSD licence.
 * Author: Tomasz Sowa <t.sowa@ttmath.org>
 */

/*
 * Copyright (c) 2018, Tomasz Sowa
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in the
 * documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 */

#ifndef headerfile_morm_postgresqlconnector
#define headerfile_morm_postgresqlconnector

#include "dbconnector.h"
#include <libpq-fe.h>
#include <string>



namespace morm
{

class PostgreSQLConnector : public DbConnector
{
public:
	PostgreSQLConnector();
	virtual ~PostgreSQLConnector();

	void set_log_queries(bool log_queries);

	bool query(const PT::TextStream & stream);
	bool query(const char * query_str);
	bool query(const std::string & query_str);

	bool query_select(const char * query_str);
	bool query_update(const char * query_str);
	bool query_insert(const char * query_str);

	bool query_select(const PT::TextStream & stream);
	bool query_update(const PT::TextStream & stream);
	bool query_insert(const PT::TextStream & stream);

	// give me a better name
	virtual size_t last_select_size();

	// give me a better name
	virtual void set_current_row_at_beginning();
	virtual void advance_current_row();


	bool is_last_result(ExecStatusType t);

	/*
	 * get column index from the last query (select)
	 * returns -1 if there is no such a column
	 *
	 */
	int get_column_index(const char * column_name);
	int get_column_index(const wchar_t * column_name);

	bool is_null(int row, int col);

	/*
	 * return an item from the last query
	 * can return a null pointer if there is no such an item in the last result
	 *
	 */
	const char * get_value(int row, int col);
	int get_value_length(int row, int col);


	void clear_result();

	void set_conn_param(const std::wstring & database, const std::wstring & user, const std::wstring & pass);
	void connect();
	void wait_for_connection();
	void close();
	bool assert_connection(bool put_log = true, bool throw_if_no_connection = true);
	void set_db_parameters();
	void log_connection_socket();

	//PGconn * GetPgConn();


protected:

	PGconn * pg_conn;
	PGresult * last_result; // can be null
	size_t last_result_rows; // how many rows in the last result query
	ExecStatusType last_status;
	bool log_queries;
	PT::TextStream stream;
	std::string query_str;
	size_t cur_row;

	std::wstring db_database;
	std::wstring db_user;
	std::wstring db_pass;


	void allocate_default_expression();

	void overwrite(PT::TextStream & stream);
	virtual const char * get_field_string_value(const wchar_t * field_name);

	const char * query_last_sequence(const wchar_t * sequence_table_name);
};

}

#endif
